#ifndef BILLYSHEET_CHARACTER_SHEET_HPP
#define BILLYSHEET_CHARACTER_SHEET_HPP

#include "characteristic/adresse.hpp"
#include "characteristic/endurance.hpp"
#include "characteristic/chance.hpp"
#include "characteristic/habilete.hpp"
#include <random>

namespace gui {
    class Gui;
}

namespace character {
    enum class classe : std::uint8_t {
        Guerrier = 0,
        Prudent = 1,
        Paysan = 2,
        Debrouillard = 3,
    };

    class CharacterSheet final {
    private:
        friend gui::Gui;

        friend class BillyObjects;

        std::mt19937_64 engine{ std::random_device{ "rdseed" }() };

        std::string caractere{};

        //TODO: bad design pour les caractéristiques. Je dois trouver autre chose.

        characteristic::Adresse adresse;

        characteristic::Endurance endurance;

        characteristic::Chance chance;

        characteristic::Habilete habilete;

        std::uint32_t health_point{ 0 };

        std::uint32_t armor{ 0 };

        std::uint32_t damage{ 0 };

        std::uint32_t critique{ 0 };

        std::uint32_t glory{ 0 };

        std::uint32_t money{ 0 };

        std::uint32_t nb_weapons{ 0 };

        std::uint32_t nb_equipments{ 0 };

        std::uint32_t nb_tools{ 0 };
    public:
        CharacterSheet() = default;

        CharacterSheet(const CharacterSheet &sheet) noexcept = default;

        CharacterSheet(CharacterSheet &&sheet) noexcept = default;

        CharacterSheet &operator=(const CharacterSheet &characterSheet) noexcept = default;

        CharacterSheet &operator=(CharacterSheet &&characterSheet) noexcept = default;

        ~CharacterSheet() noexcept = default;

        [[nodiscard]] const std::string &get_caractere() const { return caractere; }

        [[nodiscard]] const characteristic::Adresse &get_adresse() const { return adresse; }

        [[nodiscard]] const characteristic::Endurance &get_endurance() const { return endurance; }

        [[nodiscard]] const characteristic::Chance &get_chance() const { return chance; }

        [[nodiscard]] const characteristic::Habilete &get_habilete() const { return habilete; }

        [[nodiscard]] classe get_current_class() const {
            if (nb_weapons >= 2) {
                return classe::Guerrier;
            } else if (nb_equipments >= 2) {
                return classe::Prudent;
            } else if (nb_tools >= 2) {
                return classe::Paysan;
            } else {
                return classe::Debrouillard;
            }
        }

        [[nodiscard]] std::uint32_t get_health_point() const { return health_point; }

        [[nodiscard]] std::uint32_t get_armor() const { return armor; }

        [[nodiscard]] std::uint32_t get_damage() const { return damage; }

        [[nodiscard]] std::uint32_t get_glory() const { return glory; }

        [[nodiscard]] std::uint32_t get_money() const { return money; }

        [[nodiscard]] std::uint32_t get_nb_weapons() const { return nb_weapons; }

        [[nodiscard]] std::uint32_t get_nb_equipments() const { return nb_equipments; }

        [[nodiscard]] std::uint32_t get_nb_tools() const { return nb_tools; }

        friend void from_json(const json &j, CharacterSheet &billy) {
            j.at("caractere").get_to(billy.caractere);
            j.at("adresse").get_to(billy.adresse);
            j.at("endurance").get_to(billy.endurance);
            j.at("chance").get_to(billy.chance);
            j.at("habilete").get_to(billy.habilete);
            j.at("health_point").get_to(billy.health_point);
            j.at("armor").get_to(billy.armor);
            j.at("damage").get_to(billy.damage);
            j.at("glory").get_to(billy.glory);
            j.at("money").get_to(billy.money);
            j.at("nb_weapons").get_to(billy.nb_weapons);
            j.at("nb_equipments").get_to(billy.nb_equipments);
            j.at("nb_tools").get_to(billy.nb_tools);
        }
    };

    static void to_json(json &j, const CharacterSheet &billy) {
        j["caractere"] = billy.get_caractere();
        j["adresse"] = billy.get_adresse();
        j["endurance"] = billy.get_endurance();
        j["chance"] = billy.get_chance();
        j["habilete"] = billy.get_habilete();
        j["classe"] = billy.get_current_class();
        j["health_point"] = billy.get_health_point();
        j["armor"] = billy.get_armor();
        j["damage"] = billy.get_damage();
        j["glory"] = billy.get_glory();
        j["money"] = billy.get_money();
        j["nb_weapons"] = billy.get_nb_weapons();
        j["nb_equipments"] = billy.get_nb_equipments();
        j["nb_tools"] = billy.get_nb_tools();
    }
}


#endif //BILLYSHEET_CHARACTER_SHEET_HPP
